﻿using NMF.Collections.Generic;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Diagnostics;
using System.Linq;
using System.Text;

namespace NMF.Collections.ObjectModel
{
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1710:IdentifiersShouldHaveCorrectSuffix"), DebuggerDisplay("Count = {Count}"), DebuggerTypeProxy(typeof(EnumerableDebuggerProxy<>))]
    public class ObservableSet<T> : DecoratedSet<T>, ISet<T>, ICollection<T>, IEnumerable<T>, ICollection, IEnumerable, INotifyCollectionChanged
    {
        public override bool Add(T item)
        {
            if (base.Add(item))
            {
                OnInsertItem(item);
                return true;
            }
            return false;
        }

        public override void Clear()
        {
            base.Clear();
            OnClear();
        }

        public override bool Remove(T item)
        {
            if (base.Remove(item))
            {
                OnRemoveItem(item);
                return true;
            }
            return false;
        }

        public event NotifyCollectionChangedEventHandler CollectionChanged = delegate { };

        protected virtual void OnClear()
        {
            CollectionChanged(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
        }

        protected virtual void OnInsertItem(T item)
        {
            CollectionChanged(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Add, item));
        }

        protected virtual void OnRemoveItem(T item)
        {
            CollectionChanged(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Remove, item));
        }

        protected virtual void OnReplaceItem(T oldItem, T newItem)
        {
            CollectionChanged(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Replace, oldItem, newItem));
        }
    }
}
