﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Numerics;
using System.Text;

namespace NMF.Interop.Ecore
{
    public static class DefaultTypes
    {
        private static readonly EString eString = new EString() { Name = "EString", IsSerializable = true };
        private static readonly EBigInteger eBigInteger = new EBigInteger() { Name = "EBigInteger", IsSerializable = true };
        private static readonly EBoolean eBoolean = new EBoolean() { Name = "EBoolean", IsSerializable = true };
        private static readonly EByte eByte = new EByte() { Name = "EByte", IsSerializable = true };
        private static readonly EByteArray eByteArray = new EByteArray() { Name = "EByteArray", IsSerializable = true };
        private static readonly EChar eChar = new EChar() { Name = "EChar", IsSerializable = true };
        private static readonly EClassMappable eClass = new EClassMappable() { Name = "EClass", IsSerializable = false };
        private static readonly EDate eDate = new EDate() { Name = "EDate", IsSerializable = true };
        private static readonly EDouble eDouble = new EDouble() { Name = "EDouble", IsSerializable = true };
        private static readonly EFloat eFloat = new EFloat() { Name = "EFloat", IsSerializable = true };
        private static readonly EInt eInt = new EInt() { Name = "EInt", IsSerializable = true };
        private static readonly ELong eLong = new ELong() { Name = "ELong", IsSerializable = true };
        private static readonly EObjectMappable eObject = new EObjectMappable() { Name = "EObject", IsSerializable = false };
        private static readonly EShort eShort = new EShort() { Name = "EShort", IsSerializable = true };
        private static readonly EDecimal eDecimal = new EDecimal() { Name = "EDecimal", IsSerializable = true };

        public static EString EString { get { return eString; } }
        public static EBigInteger EBigInteger { get { return eBigInteger; } }
        public static EBoolean EBoolean { get { return eBoolean; } }
        public static EByte EByte { get { return eByte; } }
        public static EByteArray EByteArray { get { return eByteArray; } }
        public static EChar EChar { get { return eChar; } }
        public static EClassMappable EClass { get { return eClass; } }
        public static EDate EDate { get { return eDate; } }
        public static EDouble EDouble { get { return eDouble; } }
        public static EFloat EFloat { get { return eFloat; } }
        public static EInt EInt { get { return eInt; } }
        public static ELong ELong { get { return eLong; } }
        public static EObjectMappable EObject { get { return eObject; } }
        public static EShort EShort { get { return eShort; } }
        public static EDecimal EDecimal { get { return eDecimal; } }
    }

    public class EString : MappableDataType
    {
        public override Type Type
        {
            get { return typeof(string); }
        }

        public override object ConvertFromString(string text)
        {
            return text;
        }

        public override string ConvertToString(object input)
        {
            return input.ToString();
        }
    }

    public class EBigInteger : MappableDataType
    {
        public override Type Type
        {
            get { return typeof(BigInteger); }
        }

        public override object ConvertFromString(string text)
        {
            return BigInteger.Parse(text);
        }

        public override string ConvertToString(object input)
        {
            return input.ToString();
        }
    }

    public class EBoolean : MappableDataType
    {

        public override Type Type
        {
            get { return typeof(bool); }
        }

        public override object ConvertFromString(string text)
        {
            return bool.Parse(text);
        }

        public override string ConvertToString(object input)
        {
            return input.ToString();
        }
    }

    public class EByte : MappableDataType
    {
        public override Type Type
        {
            get { return typeof(byte); }
        }

        public override object ConvertFromString(string text)
        {
            return byte.Parse(text);
        }

        public override string ConvertToString(object input)
        {
            return input.ToString();
        }
    }

    public class EByteArray : MappableDataType
    {
        public override Type Type
        {
            get { return typeof(byte[]); }
        }

        public override object ConvertFromString(string text)
        {
            return Convert.FromBase64String(text);
        }

        public override string ConvertToString(object input)
        {
            return Convert.ToBase64String(input as byte[]);
        }
    }

    public class EChar : MappableDataType
    {
        public override Type Type
        {
            get { return typeof(char); }
        }

        public override object ConvertFromString(string text)
        {
            return text[0];
        }

        public override string ConvertToString(object input)
        {
            return input.ToString();
        }
    }

    public class EDate : MappableDataType
    {
        public override Type Type
        {
            get { return typeof(DateTime); }
        }

        public override object ConvertFromString(string text)
        {
            return DateTime.Parse(text);
        }

        public override string ConvertToString(object input)
        {
            return input.ToString();
        }
    }

    public class EDouble : MappableDataType
    {
        public override Type Type
        {
            get { return typeof(double); }
        }

        public override object ConvertFromString(string text)
        {
            return double.Parse(text);
        }

        public override string ConvertToString(object input)
        {
            return input.ToString();
        }
    }

    public class EFloat : MappableDataType
    {
        public override Type Type
        {
            get { return typeof(float); }
        }

        public override object ConvertFromString(string text)
        {
            return float.Parse(text);
        }

        public override string ConvertToString(object input)
        {
            return input.ToString();
        }
    }

    public class EInt : MappableDataType
    {
        public override Type Type
        {
            get { return typeof(int); }
        }

        public override object ConvertFromString(string text)
        {
            return int.Parse(text);
        }

        public override string ConvertToString(object input)
        {
            return input.ToString();
        }
    }

    public class ELong : MappableDataType
    {
        public override Type Type
        {
            get { return typeof(long); }
        }

        public override object ConvertFromString(string text)
        {
            return long.Parse(text);
        }

        public override string ConvertToString(object input)
        {
            return input.ToString();
        }
    }

    public class EShort : MappableDataType
    {
        public override Type Type
        {
            get { return typeof(short); }
        }

        public override object ConvertFromString(string text)
        {
            return short.Parse(text);
        }

        public override string ConvertToString(object input)
        {
            return input.ToString();
        }
    }

    public class EDecimal : MappableDataType
    {
        public override Type Type
        {
            get { return typeof(Decimal); }
        }

        public override object ConvertFromString(string text)
        {
            return decimal.Parse(text);
        }

        public override string ConvertToString(object input)
        {
            return Convert.ToString(input);
        }
    }

    public class EObjectMappable : MappableDataType
    {
        public override Type Type
        {
            get { return typeof(object); }
        }

        public override object ConvertFromString(string text)
        {
            throw new InvalidOperationException();
        }

        public override string ConvertToString(object input)
        {
            throw new InvalidOperationException();
        }
    }

    public class EClassMappable : EObjectMappable
    {
        public override Type Type
        {
            get
            {
                return typeof(Type);
            }
        }
    }
}
