﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace NMF.Interop.Ecore
{
    public abstract class EClassifier : ENamedElement, INotifyPropertyChanged
    {
        #region InstanceClassName

        private string mInstanceClassName;

        public string InstanceClassName
        {
            get
            {
                return mInstanceClassName;
            }
            set
            {
                if (mInstanceClassName != value)
                {
                    mInstanceClassName = value;
                    OnInstanceClassNameChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnInstanceClassNameChanged(EventArgs e)
        {
            EventHandler handler = InstanceClassNameChanged;
            if (handler != null) handler(this, e);
            OnPropertyChanged("InstanceClassName");
        }

        public event EventHandler InstanceClassNameChanged;

        #endregion

        #region Package

        private EPackage mPackage;

        public EPackage Package
        {
            get
            {
                return mPackage;
            }
            set
            {
                if (mPackage != value)
                {
                    var old = mPackage;
                    mPackage = value;

                    if (old != null && old.Classifiers.Contains(this)) old.Classifiers.Remove(this);
                    if (value != null && !value.Classifiers.Contains(this)) value.Classifiers.Add(this);

                    OnPackageChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnPackageChanged(EventArgs e)
        {
            EventHandler handler = PackageChanged;
            if (handler != null) handler(this, e);
            OnPropertyChanged("Package");
        }

        public event EventHandler PackageChanged;

        #endregion
        
    }
}
