﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Collections.ObjectModel;

namespace NMF.Interop.Ecore
{
    public class EOperation : ETypedElement, INotifyPropertyChanged
    {
        public EOperation()
        {
            Parameters = new EOperationParameterCollection(this);
            Exceptions = new ObservableCollection<EClass>();
        }

        public EOperationParameterCollection Parameters { get; private set; }
        public ObservableCollection<EClass> Exceptions { get; private set; }

        #region DeclaringClass

        private EClass mDeclaringClass;

        public EClass DeclaringClass
        {
            get
            {
                return mDeclaringClass;
            }
            set
            {
                if (mDeclaringClass != value)
                {
                    var old = mDeclaringClass;
                    mDeclaringClass = value;

                    if (old != null && old.Operations.Contains(this)) old.Operations.Remove(this);
                    if (value != null && !value.Operations.Contains(this)) value.Operations.Add(this);

                    OnDeclaringClassChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnDeclaringClassChanged(EventArgs e)
        {
            EventHandler handler = DeclaringClassChanged;
            if (handler != null) handler(this, e);
            OnPropertyChanged("DeclaringClass");
        }

        public event EventHandler DeclaringClassChanged;

        #endregion
        
    }
}
