﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Collections.ObjectModel;

namespace NMF.Interop.Ecore
{
    public class EPackage : ENamedElement, INotifyPropertyChanged
    {
        public EPackage()
        {
            Classifiers = new EPackageClassifierCollection(this);
            SubPackages = new EPackageSubpackageCollection(this);
        }

        #region ParentPackage

        private EPackage mParentPackage;

        public EPackage ParentPackage
        {
            get
            {
                return mParentPackage;
            }
            set
            {
                if (mParentPackage != value)
                {
                    var old = mParentPackage;
                    mParentPackage = value;

                    if (old != null && old.SubPackages.Contains(this)) old.SubPackages.Remove(this);
                    if (value != null && !value.SubPackages.Contains(this)) value.SubPackages.Add(this);

                    OnParentPackageChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnParentPackageChanged(EventArgs e)
        {
            EventHandler handler = ParentPackageChanged;
            if (handler != null) handler(this, e);
            OnPropertyChanged("ParentPackage");
        }

        public event EventHandler ParentPackageChanged;

        #endregion

        #region NsUri

        private string mNsUri;

        public string NsUri
        {
            get
            {
                return mNsUri;
            }
            set
            {
                if (mNsUri != value)
                {
                    mNsUri = value;
                    OnNsUriChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnNsUriChanged(EventArgs e)
        {
            EventHandler handler = NsUriChanged;
            if (handler != null) handler(this, e);
            OnPropertyChanged("NsUri");
        }

        public event EventHandler NsUriChanged;

        #endregion

        #region NsPrefix

        private string mNsPrefix;

        public string NsPrefix
        {
            get
            {
                return mNsPrefix;
            }
            set
            {
                if (mNsPrefix != value)
                {
                    mNsPrefix = value;
                    OnNsPrefixChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnNsPrefixChanged(EventArgs e)
        {
            EventHandler handler = NsPrefixChanged;
            if (handler != null) handler(this, e);
            OnPropertyChanged("NsPrefix");
        }

        public event EventHandler NsPrefixChanged;

        #endregion


        public EPackageClassifierCollection Classifiers { get; private set; }
        public EPackageSubpackageCollection SubPackages { get; private set; }
    }
}
