﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace NMF.Interop.Ecore
{
    public abstract class EStructuralFeature : ETypedElement, INotifyPropertyChanged
    {
        #region IsChangable

        private bool mIsChangable;

        public bool IsChangable
        {
            get
            {
                return mIsChangable;
            }
            set
            {
                if (mIsChangable != value)
                {
                    mIsChangable = value;
                    OnIsChangableChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnIsChangableChanged(EventArgs e)
        {
            EventHandler handler = IsChangableChanged;
            if (handler != null) handler(this, e);
            OnPropertyChanged("IsChangable");
        }

        public event EventHandler IsChangableChanged;

        #endregion

        #region IsVolatile

        private bool mIsVolatile;

        public bool IsVolatile
        {
            get
            {
                return mIsVolatile;
            }
            set
            {
                if (mIsVolatile != value)
                {
                    mIsVolatile = value;
                    OnIsVolatileChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnIsVolatileChanged(EventArgs e)
        {
            EventHandler handler = IsVolatileChanged;
            if (handler != null) handler(this, e);
            OnPropertyChanged("IsVolatile");
        }

        public event EventHandler IsVolatileChanged;

        #endregion

        #region IsTransient

        private bool mIsTransient;

        public bool IsTransient
        {
            get
            {
                return mIsTransient;
            }
            set
            {
                if (mIsTransient != value)
                {
                    mIsTransient = value;
                    OnIsTransientChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnIsTransientChanged(EventArgs e)
        {
            EventHandler handler = IsTransientChanged;
            if (handler != null) handler(this, e);
            OnPropertyChanged("IsTransient");
        }

        public event EventHandler IsTransientChanged;

        #endregion

        #region DefaultValue

        private string mDefaultValue;

        public string DefaultValue
        {
            get
            {
                return mDefaultValue;
            }
            set
            {
                if (mDefaultValue != value)
                {
                    mDefaultValue = value;
                    OnDefaultValueChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnDefaultValueChanged(EventArgs e)
        {
            EventHandler handler = DefaultValueChanged;
            if (handler != null) handler(this, e);
            OnPropertyChanged("DefaultValue");
        }

        public event EventHandler DefaultValueChanged;

        #endregion

        #region IsUnsettable

        private bool mIsUnsettable;

        public bool IsUnsettable
        {
            get
            {
                return mIsUnsettable;
            }
            set
            {
                if (mIsUnsettable != value)
                {
                    mIsUnsettable = value;
                    OnIsUnsettableChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnIsUnsettableChanged(EventArgs e)
        {
            EventHandler handler = IsUnsettableChanged;
            if (handler != null) handler(this, e);
            OnPropertyChanged("IsUnsettable");
        }

        public event EventHandler IsUnsettableChanged;

        #endregion

        #region IsDerived

        private bool mIsDerived;

        public bool IsDerived
        {
            get
            {
                return mIsDerived;
            }
            set
            {
                if (mIsDerived != value)
                {
                    mIsDerived = value;
                    OnIsDerivedChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnIsDerivedChanged(EventArgs e)
        {
            EventHandler handler = IsDerivedChanged;
            if (handler != null) handler(this, e);
            OnPropertyChanged("IsDerived");
        }

        public event EventHandler IsDerivedChanged;

        #endregion

        #region DeclaringClass

        private EClass mDeclaringClass;

        public EClass DeclaringClass
        {
            get
            {
                return mDeclaringClass;
            }
            set
            {
                if (mDeclaringClass != value)
                {
                    var old = mDeclaringClass;
                    mDeclaringClass = value;

                    if (old != null && old.Features.Contains(this)) old.Features.Remove(this);
                    if (value != null && !value.Features.Contains(this)) value.Features.Add(this);

                    OnDeclaringClassChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnDeclaringClassChanged(EventArgs e)
        {
            EventHandler handler = DeclaringClassChanged;
            if (handler != null) handler(this, e);
            OnPropertyChanged("DeclaringClass");
        }

        public event EventHandler DeclaringClassChanged;

        #endregion
        
    }
}
