﻿using NMF.Transformations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Collections;
using NMF.Transformations.Core;

namespace NMF.Interop.Ecore.Transformations
{
    public class EcoreCopyTransformationRule<T> : TransformationRule<T,T>
        where T : class
    {
        public override void Transform(T input, T output, ITransformationContext context)
        {
            if (Transformation is IEcoreTransformation)
            {
                var rulesDict = ((IEcoreTransformation)Transformation).EcoreRules;
                foreach (PropertyDescriptor pd in TypeDescriptor.GetProperties(typeof(T)))
                {
                    if (pd.IsReadOnly)
                    {
                        if (typeof(IEnumerable).IsAssignableFrom(pd.PropertyType))
                        {
                            Type iCollection = pd.PropertyType.GetInterfaces().FirstOrDefault
                                (t => t.IsGenericType && t.GetGenericTypeDefinition() == typeof(ICollection<>));
                            if (iCollection != null)
                            {
                                Type collType = iCollection.GetGenericArguments()[0];
                                if (rulesDict.ContainsKey(collType))
                                {
                                    AddRange(pd.GetValue(output), (pd.GetValue(input) as IEnumerable)
                                        .Cast<object>()
                                        .Select(o => context.Trace.ResolveIn(rulesDict[collType], new object[] { o })),
                                        iCollection);
                                }
                                else
                                {
                                    AddRange(pd.GetValue(output), pd.GetValue(input) as IEnumerable, iCollection);
                                }
                            }
                        }
                    }
                    else
                    {
                        if (rulesDict.ContainsKey(pd.PropertyType))
                        {
                            pd.SetValue(output, context.Trace.ResolveIn(rulesDict[pd.PropertyType], new object[] { pd.GetValue(input) }));
                        }
                        else
                        {
                            pd.SetValue(output, pd.GetValue(input));
                        }
                    }
                }
            }
            else
            {
                throw new InvalidOperationException("An EcoreCopyTransformationRule may only be used within an EcoreTransformation");
            }
        }

        private void AddRange(object outCollection, IEnumerable enumerable, Type iCollection)
        {
            if (enumerable != null)
            {
                foreach (var item in enumerable)
                {
                    iCollection.InvokeMember("Add", System.Reflection.BindingFlags.Public | System.Reflection.BindingFlags.Instance, null, outCollection, new object[] { item });
                }
            }
        }
    }
}
