﻿using NMF.Transformations;
using NMF.Transformations.Core;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;

namespace NMF.Interop.Ecore.Transformations
{
    public abstract class EcoreTransformation<T> : ReflectiveTransformation, IEcoreTransformation
    {
        private Dictionary<Type, GeneralTransformationRule> rules;

        protected override IEnumerable<GeneralTransformationRule> CreateCustomRules()
        {
            var rules = new Dictionary<Type, GeneralTransformationRule>();

            RegisterTypes(typeof(T), rules);

            this.rules = rules;

            return rules.Values;
        }

        IDictionary<Type, GeneralTransformationRule> IEcoreTransformation.EcoreRules { get { return rules; } }

        protected abstract GeneralTransformationRule CreateRuleForType(Type type);

        private GeneralTransformationRule RegisterTypes(Type type, Dictionary<Type, GeneralTransformationRule> rules)
        {
            GeneralTransformationRule rule;
            if (!rules.TryGetValue(type, out rule))
            {
                rule = CreateRuleForType(type);
                rules.Add(type, rule);
                if (type.IsInterface)
                {
                    foreach (var childInterface in type.GetInterfaces())
                    {
                        var baseRule = RegisterTypes(type, rules);
                        rule.MarkInstantiatingFor(baseRule);
                    }
                }
                else
                {
                    if (type.BaseType != typeof(object))
                    {
                        var baseRule = RegisterTypes(type.BaseType, rules);
                        rule.MarkInstantiatingFor(baseRule);
                    }
                }

                foreach (KnownTypeAttribute att in type.GetCustomAttributes(typeof(KnownTypeAttribute), false))
                {
                    var derivedRule = RegisterTypes(att.Type, rules);
                    derivedRule.MarkInstantiatingFor(rule);
                }


                //foreach (PropertyDescriptor pd in TypeDescriptor.GetProperties(type))
                //{
                //    var att = pd.Attributes[typeof(ContainmentAttribute)];
                //    if (att != null)
                //    {
                //        if (pd.IsReadOnly)
                //        {
                //            if (typeof(IEnumerable).IsAssignableFrom(pd.PropertyType))
                //            {
                //                Type iCollection = pd.PropertyType.GetInterfaces().FirstOrDefault
                //                    (t => t.IsGenericType && t.GetGenericTypeDefinition() == typeof(ICollection<>));
                //                if (iCollection != null) {
                //                        Type collType = iCollection.GetGenericArguments()[0];
                //                        var dependent = RegisterTypes(collType, rules);
                //                        rule.RequireMany(dependent, objects => 
                //                            (pd.GetValue(objects[0]) as IEnumerable)
                //                                .Cast<object>()
                //                                .Select(o => new object[] { o }));
                //                }
                //            }
                //        }
                //        else
                //        {
                //            var dependent = RegisterTypes(pd.PropertyType, rules);
                //            rule.Require(dependent, o => new object[] { pd.GetValue(o[0]) });
                //        }
                //    }
                //}
            }
            return rule;
        }
    }
}
