﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace NMF.Models
{
    public interface IModelElement : INotifyPropertyChanged, IDataErrorInfo
    {
        IEnumerable<IModelElement> ChildElements { get; }

        bool Validate();
    }

    public abstract class ModelElement : IModelElement
    {
        public virtual bool Validate()
        {
            return GetValidationError(null) == null;   
        }

        protected virtual void OnPropertyChanged(string property)
        {
            PropertyChanged(this, new PropertyChangedEventArgs(property));
        }

        public event PropertyChangedEventHandler PropertyChanged = delegate { };

        public string Error
        {
            get { return GetValidationError(null); }
        }

        string IDataErrorInfo.this[string columnName]
        {
            get { return GetValidationError(columnName); }
        }

        protected virtual string GetValidationError(string property)
        {
            return null;
        }

        public IEnumerable<IModelElement> ChildElements
        {
            get { return Enumerable.Empty<IModelElement>(); }
        }
    }

}
