﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Xml;

namespace NMF.Serialization
{

    public class XmlPropertySerializationInfo : IPropertySerializationInfo
    {

        private PropertyDescriptor prop;

        public XmlPropertySerializationInfo(PropertyDescriptor prop)
        {
            this.prop = prop;
        }

        public PropertyDescriptor Descriptor
        {
            get
            {
                return prop;
            }
        }

        public TypeConverter Converter
        {
            get
            {
                if (CustomConverter != null) return CustomConverter;
                return Descriptor.Converter;
            }
        }

        public TypeConverter CustomConverter
        {
            get;
            set;
        }

        public bool ShallCreateInstance
        {
            get;
            set;
        }

        public string ElementName
        {
            get;
            set;
        }

        public string Namespace
        {
            get;
            set;
        }

        public string NamespacePrefix
        {
            get;
            set;
        }

        public virtual object GetValue(object obj, ISerializationContext context)
        {
            return Descriptor.GetValue(obj);
        }

        public virtual void SetValue(object obj, object value, ISerializationContext context)
        {
            Descriptor.SetValue(obj, value);
        }

        public virtual bool ShouldSerializeValue(object obj)
        {
            return Descriptor.ShouldSerializeValue(obj);
        }

        public bool IsIdentifier
        {
            get;
            set;
        }

        public XmlIdentificationMode IdentificationMode
        {
            get;
            set;
        }

        public bool IsStringConvertible
        {
            get { return Converter != null && Converter.CanConvertFrom(typeof(string)) && Converter.CanConvertTo(typeof(string)); }
        }

        public object ConvertFromString(string text)
        {
            return Converter.ConvertFromInvariantString(text);
        }

        public string ConvertToString(object input)
        {
            return Converter.ConvertToInvariantString(input);
        }

        public bool IsReadOnly
        {
            get { return Descriptor.IsReadOnly; }
        }


        public ITypeSerializationInfo PropertyType
        {
            get;
            set;
        }
    }
}
