﻿using System;
using System.Collections.Generic;
using System.Text;

namespace NMF.Serialization
{
    public class XmlSerializationContext : ISerializationContext
    {
        public XmlSerializationContext(ISerializer serializer, object root)
        {
            Serializer = serializer;
            Root = root;
        }

        private Dictionary<ITypeSerializationInfo, Dictionary<string, object>> idStore = new Dictionary<ITypeSerializationInfo, Dictionary<string, object>>();

        public virtual void RegisterId(string id, object value)
        {
            if (value != null)
            {
                var type = Serializer.GetSerializationInfo(value.GetType(), false);
                InsertIntoIdStore(id, value, type);
            }
        }

        private void InsertIntoIdStore(string id, object value, ITypeSerializationInfo type)
        {
            Dictionary<string, object> dict;
            if (!idStore.TryGetValue(type, out dict))
            {
                dict = new Dictionary<string, object>();
                idStore.Add(type, dict);
            }
            if (!dict.ContainsKey(id))
            {
                dict.Add(id, value);
            }
            else
            {
                dict[id] = value;
            }
            if (type.BaseTypes != null)
            {
                foreach (var baseType in type.BaseTypes)
                {
                    InsertIntoIdStore(id, value, baseType);
                }
            }
        }

        public virtual bool ContainsId(string id, ITypeSerializationInfo type)
        {
            if (idStore.ContainsKey(type))
            {
                return idStore[type].ContainsKey(id);
            }
            else
            {
                return false;
            }
        }

        public virtual object Resolve(string id, ITypeSerializationInfo type)
        {
            Dictionary<string, object> dict;
            object obj;
            if (idStore.TryGetValue(type, out dict) && dict.TryGetValue(id, out obj))
            {
                return obj;
            }
            else
            {
                return null;
            }
        }

        public object Root
        {
            get;
            private set;
        }

        public ISerializer Serializer
        {
            get;
            private set;
        }
    }
}
