﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;

using NMF.Tests;

using NMF.Transformations.Linq;
using System.Collections.Generic;
using System.Collections.ObjectModel;

using System.Linq;

namespace NMF.Transformations.Tests.UnitTests.Relational
{
    [TestClass]
    public class GrowableSourceTest
    {
        private GrowableSource<string> source = new GrowableSource<string>(null);

        [TestMethod]
        public void Transformations_Relational_GrowableSource_Enum()
        {
            source.AssertEmpty();
            source.Add("a");
            source.AssertContainsOnly("a");
            source.Add("b");
            source.AssertContainsOnly("a", "b");
            source.Add("c");
            source.AssertContainsOnly("a", "b", "c");
        }

        [TestMethod]
        public void Transformations_Relational_GrowableSource_NextItem()
        {
            source.AssertEmpty();
            bool eventCalled = false;

            source.NextItem += item =>
            {
                Assert.AreEqual(source.Last(), item);
                eventCalled = !eventCalled;
            };

            source.Add("a");
            source.AssertContainsOnly("a");
            Assert.IsTrue(eventCalled);
            eventCalled = false;

            source.Add("b");
            source.AssertContainsOnly("a", "b");
            Assert.IsTrue(eventCalled);
            eventCalled = false;

            source.Add("c");
            source.AssertContainsOnly("a", "b", "c");
            Assert.IsTrue(eventCalled);
        }

        [TestMethod]
        public void Transformations_Relational_GrowableSource_Dispose1()
        {
            source.Add("a");
            source.Dispose();
        }

        [TestMethod]
        public void Transformations_Relational_GrowableSource_Observable()
        {
            var list = new ObservableCollection<string>();
            source = new GrowableSource<string>(list);
            list.AssertEmpty();
            bool eventCalled = false;

            source.NextItem += item =>
            {
                Assert.AreEqual(list[list.Count - 1], item);
                eventCalled = !eventCalled;
            };

            list.Add("a");
            source.AssertContainsOnly("a");
            Assert.IsTrue(eventCalled);
            eventCalled = false;

            list.Add("b");
            source.AssertContainsOnly("a", "b");
            Assert.IsTrue(eventCalled);
            eventCalled = false;

            list.Add("c");
            source.AssertContainsOnly("a", "b", "c");
            Assert.IsTrue(eventCalled);
        }

        [TestMethod]
        public void Transformations_Relational_GrowableSource_Dispose2()
        {
            source.Dispose();

            var list = new ObservableCollection<string>();
            source = new GrowableSource<string>(list);
            list.AssertEmpty();
            bool eventCalled = false;

            source.NextItem += item =>
            {
                Assert.AreEqual(list[list.Count - 1], item);
                eventCalled = !eventCalled;
            };

            list.Add("a");
            source.AssertContainsOnly("a");
            Assert.IsTrue(eventCalled);
            eventCalled = false;

            list.Add("b");
            source.AssertContainsOnly("a", "b");
            Assert.IsTrue(eventCalled);
            eventCalled = false;

            source.Dispose();

            list.Add("c");
            Assert.IsFalse(eventCalled);
        }
    }
}
