﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NMF.Transformations.Linq
{
    /// <summary>
    /// Represents the default implementation for the Where operation
    /// </summary>
    /// <typeparam name="T">The inner type of the filtered monad instance</typeparam>
    internal class FilteredSource<T> : IGrowableSource<T>
    {
        internal FilteredSource(IGrowableSource<T> source, Func<T, bool> filter)
        {
            this.Source = source;
            this.Filter = filter;

            source.NextItem += source_NextItem;
        }

        void source_NextItem(T obj)
        {
            if (Filter(obj) && NextItem != null) { NextItem(obj); }
        }

        public IGrowableSource<T> Source { get; private set; }
        public Func<T, bool> Filter { get; private set; }

        public event Action<T> NextItem;

        public void Dispose()
        {
            Source.NextItem -= source_NextItem;
            Source.Dispose();
        }

        public IEnumerator<T> GetEnumerator()
        {
            return ((IEnumerable<T>)Source).Where(Filter).GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }
    }
}
