﻿using NMF.Transformations.Core;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NMF.Transformations.Linq
{
    /// <summary>
    /// This class represents the pattern context for a relational pattern applied to a transformation rule with one input argument
    /// </summary>
    /// <typeparam name="TIn">The type of the input argument of the parented transformation rule</typeparam>
    internal class RelationalPatternContext<TIn> : ITransformationPatternContext
        where TIn : class
    {
        public IGrowableSource<TIn> Source { get; set; }
        public ITransformationContext Context { get; set; }
        public GeneralTransformationRule<TIn> TargetRule { get; set; }

        public RelationalPatternContext(IGrowableSource<TIn> relationalSource, GeneralTransformationRule<TIn> targetRule, ITransformationContext context)
        {
            this.Source = relationalSource;
            this.Source.NextItem += Source_NextItem;
            this.Context = context;
            this.TargetRule = targetRule;
        }

        void Source_NextItem(TIn obj)
        {
            Context.CallTransformation(TargetRule, obj);
        }

        public void Finish()
        {
            Source.Dispose();
        }
    }

    /// <summary>
    /// This class represents the pattern context for a relational pattern applied to a transformation rule with two input arguments
    /// </summary>
    /// <typeparam name="TIn1">The type of the first input argument of the parented transformation rule</typeparam>
    /// <typeparam name="TIn2">The type of the second input argument of the parented transformation rule</typeparam>
    internal class RelationalPatternContext<TIn1, TIn2> : ITransformationPatternContext
        where TIn1 : class
        where TIn2 : class
    {
        public IGrowableSource<Tuple<TIn1, TIn2>> Source { get; set; }
        public ITransformationContext Context { get; set; }
        public GeneralTransformationRule<TIn1, TIn2> TargetRule { get; set; }

        public RelationalPatternContext(IGrowableSource<Tuple<TIn1, TIn2>> relationalSource, GeneralTransformationRule<TIn1, TIn2> targetRule, ITransformationContext context)
        {
            this.Source = relationalSource;
            this.Source.NextItem += Source_NextItem;
            this.Context = context;
            this.TargetRule = targetRule;
        }

        void Source_NextItem(Tuple<TIn1, TIn2> obj)
        {
            if (obj != null)
            Context.CallTransformation(TargetRule, obj.Item1, obj.Item2);
        }

        public void Finish()
        {
            Source.Dispose();
        }
    }
}
