﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NMF.Transformations.Linq
{
    internal class SelectSource<T1, T2> : IGrowableSource<T2>
    {
        public SelectSource(IGrowableSource<T1> items, Func<T1, T2> selector)
        {
            this.Source = items;
            this.Selector = selector;

            items.NextItem += items_NextItem;
        }

        void items_NextItem(T1 obj)
        {
            if (NextItem != null)
            {
                NextItem(Selector(obj));
            }
        }

        public IGrowableSource<T1> Source { get; private set; }
        public Func<T1, T2> Selector { get; private set; }

        public event Action<T2> NextItem;

        public void Dispose()
        {
            Source.Dispose();

            Source.NextItem -= items_NextItem;
        }

        public IEnumerator<T2> GetEnumerator()
        {
            return ((IEnumerable<T1>)Source).Select(Selector).GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }
    }
}
