﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NMF.Transformations.Linq
{
    class SimpleManySource<TItem, TElement, TTarget> : IGrowableSource<TTarget>
    {
        public SimpleManySource(IGrowableSource<TItem> source, Func<TItem, IEnumerable<TElement>> elements, Func<TItem, TElement, TTarget> selector)
        {
            Source = source;
            Elements = elements;
            Selector = selector;

            source.NextItem += source_NextItem;
        }

        void source_NextItem(TItem item)
        {
            foreach (var element in Elements(item))
            {
                NextItem(Selector(item, element));
            }
        }   

        public IGrowableSource<TItem> Source { get; private set; }
        public Func<TItem, IEnumerable<TElement>> Elements { get; private set; }
        public Func<TItem, TElement, TTarget> Selector { get; private set; }

        public void Dispose()
        {
            Source.NextItem -= source_NextItem;
            Source.Dispose();
        }

        public event Action<TTarget> NextItem = delegate { };

        public IEnumerator<TTarget> GetEnumerator()
        {
            return ((IEnumerable<TItem>)Source).SelectMany(Elements, Selector).GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }
    }
}
